@@ -28,12 +28,11 @@ import pandas as pd
 from k_means_constrained import KMeansConstrained
 from PIL import Image, ImageColor, ImageDraw, ImageFilter, ImageFont
 
-from query_integral_image import query_integral_image
-from tokenization import process_tokens, unigrams_and_bigrams, word_tokenize
-from processing import plot_TSNE, embed_w2v, generate_kmeans_frequencies, kmeans_cluster, rank_kmeans
+from .query_integral_image import query_integral_image
+from .tokenization import process_tokens, unigrams_and_bigrams, word_tokenize
+from .processing import plot_TSNE, embed_w2v, kmeans_cluster
 
 FILE = os.path.dirname(__file__)
-# FONT_PATH = os.environ.get('FONT_PATH', os.path.join(FILE, 'DroidSansMono.ttf'))
 FONT_PATH = os.environ.get('FONT_PATH', os.path.join(FILE, 'THSarabun.ttf'))
 STOPWORDS = set(map(str.strip, open(os.path.join(FILE, 'stopwords')).readlines()))
 STOPWORDS_TH = set(map(str.strip, open(os.path.join(FILE, 'thstopwords')).readlines()))
@@ -401,23 +400,8 @@ class WordCloud(object):
             self.width = mask.shape[1]
             self.height = mask.shape[0]
 
-    def fit_words(self, frequencies):
-        """Create a word_cloud from words and frequencies.
 
-        Alias to generate_from_frequencies.
-
-        Parameters
-        ----------
-        frequencies : dict from string to float
-            A contains words and associated frequency.
-
-        Returns
-        -------
-        self
-        """
-        return self.generate_from_frequencies(frequencies)
-
-    def generate_from_frequencies(self, frequencies, max_font_size=None, tsne_plot=None):  # noqa: C901
+    def generate_from_frequencies(self, frequencies, max_font_size=None, tsne_plot=None, plot_now=True):  # noqa: C901
         """Create a word_cloud from words and frequencies.
 
         Parameters
@@ -493,7 +477,7 @@ class WordCloud(object):
                 font_size = self.height
             else:
                 self.generate_from_frequencies(dict(frequencies[:2]),
-                                               max_font_size=self.height)
+                                               max_font_size=self.height, plot_now=False, tsne_plot=tsne_plot)
                 # find font sizes
                 sizes = [x[1] for x in self.layout_]
                 try:
@@ -620,7 +604,12 @@ class WordCloud(object):
                                 orientations, colors))
         ## edit
         # print(collect_font_size)
-        
+        if plot_now:
+            plt.style.use('ggplot')
+            plt.figure(figsize=(9.6,4.8))
+            plt.imshow(self,interpolation="bilinear")
+            plt.axis('off')
+            plt.show()
         return self
 
     def process_text(self, text, most_common=60):
@@ -682,7 +671,7 @@ class WordCloud(object):
 
         return word_counts
 
-    def generate_from_text(self, text, tsne_plot=None, kmeans=False):
+    def generate_from_text(self, text, tsne_plot=None, kmeans=False, plot_now=True):
         """Generate wordcloud from text.
 
         The input "text" is expected to be a natural text. If you pass a sorted
@@ -695,32 +684,98 @@ class WordCloud(object):
         self
         """
         words = self.process_text(text)
-        if kmeans==True:
-            model = embed_w2v(words,lang=self.language)
-            kmeans_freq = generate_kmeans_frequencies(model, words, NUM_CLUSTERS=6, size_min=10,size_max=12)
-            clus = kmeans_cluster(model)
-            rank = rank_kmeans(kmeans_freq, rank_type='big')
-            
-        ###
-        self.generate_from_frequencies(words, tsne_plot)
+        if kmeans:
+            if not plot_now:
+                print('kmeans option overrides plot_now option')
+            self.generate_kmeans_cloud(words)
+        else:
+            self.generate_from_frequencies(words, tsne_plot, plot_now=plot_now)
         return self
 
-    def generate(self, text, tsne_plot=None, kmeans=False):
-        """Generate wordcloud from text.
 
-        The input "text" is expected to be a natural text. If you pass a sorted
-        list of words, words will appear in your output twice. To remove this
-        duplication, set ``collocations=False``.
-
-        Alias to generate_from_text.
-
-        Calls process_text and generate_from_frequencies.
+    def generate_kmeans_frequencies(self, model, word_count, label=None, NUM_CLUSTERS=6, size_min=10, size_max=12):
+        """
+        Parameters
+        ----------
+        model : gensim.models.KeyedVector or list of tuple of (str, list[str])
+            word vector model (must come with 'labels') or list of tuple of word and word vectors (no 'labels' needed)
 
+        label : list of str (optional)
+            words that we focused on; only in case of the 'model' is a whole word vector model.
+        
         Returns
         -------
-        self
+        List from str to tuple of floats, contains coordinates of words.
         """
-        return self.generate_from_text(text, tsne_plot, kmeans)
+
+        if label is None:
+            label = list(map(lambda x: x[0], model))
+        #   tokens = list(map(lambda x: x[1], model))
+        # else:
+        #   tokens = [model[word] for word in labels]
+
+        df = pd.DataFrame(data={'word': label, 'cluster': kmeans_cluster(model,NUM_CLUSTERS,size_min,size_max)})
+        df['word_count'] = df['word'].map(word_count)
+        k_means_freq = []
+        
+        for i in range(NUM_CLUSTERS):
+            clus_i = df.loc[df['cluster'] == i]
+            clus_i['total'] = clus_i['word_count'].sum()
+            clus_i_dict = {}
+            for _, row in clus_i.iterrows():
+                clus_i_dict[row['word']] = row['word_count']/row['total']
+            sorted_dict_i = sorted(clus_i_dict.items(), key=lambda item: item[1],reverse=True)[:10]
+
+            lst = []
+            for k,v in sorted_dict_i:
+                lst.append((k,v))
+            k_means_freq.append((i,lst))
+
+        return k_means_freq
+
+
+    def generate_kmeans_cloud(self, words):
+        
+        model = embed_w2v(words, lang=self.language)
+        kmeans_freq = self.generate_kmeans_frequencies(model, words, NUM_CLUSTERS=6, size_min=10,size_max=12)
+        clus = kmeans_cluster(model)
+
+        lis = []
+        clouds = []
+        for i in range(6):
+            cloud = WordCloud(font_path=self.font_path,
+                            background_color='white',
+                            width=self.width//3,
+                            height=self.height//2,
+                            max_words=20,
+                            max_font_size=self.max_font_size,
+                            colormap='tab10',
+                            color_func=lambda *args, **kwargs: "black",
+                            prefer_horizontal=1.0)
+            clouds.append(cloud)
+
+        fig, axes = plt.subplots(2,3, figsize=(32,18), sharex=True, sharey=True)
+
+        for i, ax in enumerate(axes.flatten()):
+            fig.add_subplot(ax)
+                
+            fig.tight_layout(rect=[0, 0.03, 1, 0.95])
+                
+            topic_words = dict(kmeans_freq[i][1]) #list of (words, freq)
+            clouds[i].generate_from_frequencies(topic_words, plot_now=False) # set topic
+            lis.append(clouds[i])
+            print(f'generating cloud {i}')
+            plt.gca().imshow(clouds[i],aspect="auto",interpolation = "bilinear")  # blur it right here
+            plt.gca().axis('off')
+
+        #Users have no choice but plotting now
+        # plt.subplots_adjust(wspace=0, hspace=0)
+        # plt.axis('off')
+        # plt.margins(x=0, y=0)
+        # plt.tight_layout()
+        # plt.figure(figsize=(32,18))
+        # plt.show()
+        return lis
 
     def _check_generated(self):
         """Check if ``layout_`` was computed, otherwise raise error."""
@@ -1123,39 +1178,4 @@ class WordCloud(object):
         return Image.fromarray(ret)
 
 
-def generate_kmeans_cloud(freq, max_font, fix_width, fix_height, font_path=None):
-    lis = []
-    topics = freq
-    clouds = []
-    for i in range(6):
-        cloud = WordCloud(font_path=font_path,
-                        background_color='white',
-                        width=fix_width,
-                        height=fix_height,
-                        max_words=20,
-                        colormap='tab10',
-                        color_func=lambda *args, **kwargs: "black",
-                        prefer_horizontal=1.0)
-        clouds.append(cloud)
-
-    fig2, axes2 = plt.subplots(2,3, figsize=(32,18), sharex=True, sharey=True)
-
-    for i, ax in enumerate(axes2.flatten()):
-        fig2.add_subplot(ax)
-            
-        fig2.tight_layout(rect=[0, 0.03, 1, 0.95])
-            
-        topic_words = dict(topics[i][1])
-        clouds[i].generate_from_frequencies(topic_words, max_font_size= max_font) # set topic
-        lis.append(clouds[i])
-        plt.gca().imshow(clouds[i],aspect="auto",interpolation = "bilinear")  # blur it right here
-        plt.gca().axis('off')
-
-    plt.subplots_adjust(wspace=0, hspace=0)
-    plt.axis('off')
-    plt.margins(x=0, y=0)
-    plt.tight_layout()
-    plt.figure(figsize=(32,18))
-    plt.show()
-    return lis
 
